#ifndef DOCVIEW_H
#define DOCVIEW_H

#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>
#include "patheditor.h"
#include "gdtext.h"
#include "propseditor.h"

#define PREPAREUNDO(target, method) ({\
[undoManager beginUndoGrouping]; \
[[undoManager prepareWithInvocationTarget: target] method]; \
[undoManager endUndoGrouping]; \
})

@class DocWin;

@interface DocView : NSView
{
	NSMutableArray *objects, *delObjects;
	int edind;
	BOOL shiftclick, altclick, ctrlclick;
	
	DocWin *myWin;
	NSString *docName;
	
	NSUndoManager *undoManager;
	NSInvocation *doItAgain;
	
	NSRect pageRect, a4Rect, zmdRect;
	int zIndex;
	float zFactor;
}

- (id)initInWin:(DocWin *)aWin;

- (void)doSave;

- (void)doSaveAs;

- (void)doPrint;

- (NSArray *)usedFonts;

- (BOOL)createObjectsFromDictionaryAtPath:(NSString *)aPath;

- (NSString *)docName;

- (void)addPathEditor;

- (void)addTextAtPoint:(NSPoint)p;

- (NSArray *)duplicateObjects:(NSArray *)objs andMoveTo:(NSPoint)p;

- (void)deleteSelectedObjects;

- (void)undoDeleteObjects;

- (void)startDrawingAtPoint:(NSPoint)p;

- (void)selectObjectAtPoint:(NSPoint)p;

- (void)editPathAtPoint:(NSPoint)p;

- (void)editTextAtPoint:(NSPoint)p;

- (void)editSelectedText;

- (void)moveSelectedObjects:(NSArray *)objs startingPoint:(NSPoint)startp;

- (void)undoMoveObjects:(NSArray *)objs moveBackTo:(NSPoint)p;

- (BOOL)moveControlPointOfEditor:(BezierPathEditor *)editor toPoint:(NSPoint)pos;

- (BOOL)moveBezierHandleOfEditor:(BezierPathEditor *)editor toPoint:(NSPoint)pos;

- (void)subdividePathAtPoint:(NSPoint)p splitIt:(BOOL)split;

- (void)editObjectsProperties;

- (void)moveSelectedObjectsToFront;

- (void)moveSelectedObjectsToBack;

- (void)unselectOtherObjects:(id)anObject;

- (void)zoomOnPoint:(NSPoint)p zoomOut:(BOOL)isout;

- (void)movePageFromHandPoint:(NSPoint)handpos;

- (void)doCut;

- (void)doCopy;

- (void)doPaste;

- (void)doUndo;

- (void)doRedo;

- (void)prepareDoItAgainWithSelector:(SEL)selector 
								owner:(id)owner target:(id)target , ...;

- (void)verifyModifiersOfEvent:(NSEvent *)theEvent;

- (BOOL)shiftclick;

- (BOOL)altclick;

- (BOOL)ctrlclick;

@end

#endif // DOCVIEW_H						
						
