#include "docview.h"
#include "gdraw.h"
#include "docwin.h"
#include "functions.h"
#include "texteditor.h"

float zFactors[9] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6, 8};

@implementation DocView

- (id)initInWin:(DocWin *)aWin
{
	pageRect = NSMakeRect(0, 0, 695, 942);
	a4Rect = NSMakeRect(50, 50, 595, 842);
	zmdRect = NSMakeRect(50, 50, 595, 842);
	 
	self = [super initWithFrame: pageRect];
	if(self) {

		NSImage *img = [NSImage imageNamed: @"x.tiff"];
		NSCursor *cur = [[NSCursor alloc] initWithImage: img hotSpot: NSMakePoint(0, 15)];
		[cur setOnMouseEntered: YES];
		[cur setOnMouseExited: YES];
		[self addTrackingRect: [self frame] 
						  			owner: cur userData: NULL assumeInside: YES];
	
		myWin = aWin;
		docName = nil;
		objects = [[NSMutableArray alloc] initWithCapacity: 1];
		delObjects = [[NSMutableArray alloc] initWithCapacity: 1];
		undoManager = [[NSUndoManager alloc] init];
		doItAgain = nil;
		shiftclick = NO;
		altclick = NO;
		ctrlclick = NO;
		zIndex = 2;
		zFactor = zFactors[zIndex];
	}
	return self;
}

- (void)dealloc
{
	[objects release];
	[delObjects release];
	[undoManager release];
	[super dealloc];
}

- (void)doSave
{
	NSMutableDictionary *objsdict;
	NSString *str;
	id obj;
	int i, p = 0, t = 0;

	if(!docName)
		[self doSaveAs];
	
	if(docName) {
		objsdict = [NSMutableDictionary dictionaryWithCapacity: 1];
		for(i = 0; i < [objects count]; i++) {
			obj = [objects objectAtIndex: i];
			if([obj isKindOfClass: [BezierPathEditor class]]) {
				str = [NSString stringWithFormat: @"path%i", p];
				p++;
			} else {
				str = [NSString stringWithFormat: @"text%i", t];	
				t++;
			}
			[objsdict setObject: [obj objectDescription] forKey: str];
		}
		[objsdict writeToFile: docName atomically: NO];
		[myWin setSaved: YES];
	}
}

- (void)doSaveAs
{
	NSSavePanel *savePanel;
	int result;
	BOOL exists, isdir;
	
	savePanel = [NSSavePanel savePanel];
	[savePanel setTitle: @"save"];
	[savePanel setTreatsFilePackagesAsDirectories: NO];
	[savePanel setRequiredFileType: @"gdr"];
	result = [savePanel runModalForDirectory: [[GDraw sharedgdraw] workPath] file: @""];
	if(result != NSOKButton)
		return;

	if(docName)
		[docName release];
	docName = [[NSString alloc] initWithString: [savePanel filename]];
	exists = [[NSFileManager defaultManager] fileExistsAtPath: docName isDirectory: &isdir];
		
	if(exists) {	
		result = NSRunAlertPanel(@"Alert", [NSString stringWithFormat:
									@"%@ already exists! Do you want ovewrite it?", 
										[docName lastPathComponent]], @"Ok", @"No", nil);
		if(result != NS_ALERTDEFAULT) {
			[docName release];
			docName = nil;
			return;
		}
		if(isdir) {
			NSRunAlertPanel(@"Alert", [NSString stringWithFormat:
				@"%@ is a directory!", [docName lastPathComponent]], @"Ok", nil, nil);
			[docName release];
			docName = nil;
			return;
		}
	}
	
	[myWin setName: [docName lastPathComponent]];
	[[GDraw sharedgdraw] setWorkPath: [docName stringByDeletingLastPathComponent]];
	[self doSave];
}

- (void)doPrint
{
	NSString *str, *fpath, *prstr, *fname;
	NSArray *fnames;
	id obj;
	int i;

	prstr = [NSString stringWithFormat: @"%%!PS-Adobe-2.0 EPSF-1.2
%%%%Creator: GDraw 0.1
%%%%For: %@
%%%%Title: %@
%%%%CreationDate: %@
%%%%Pages: 1
%%%%Copies: 1
%%%%DocumentSuppliedResources: procset GDraw_Procset 0 0
", NSFullUserName(), docName, [[NSDate date] description]];

	fnames = [self usedFonts];
	if([fnames count]) {
		for(i = 0; i < [fnames count]; i++)
			prstr = [prstr stringByAppendingFormat: @"%%%%+ font %@\n", 
																[fnames objectAtIndex: i]];
	}
	
	prstr = [prstr stringByAppendingFormat: @"%%%%BoundingBox: %i %i %i %i\n", 
										(int)a4Rect.origin.x, (int)a4Rect.origin.y, 
									(int)a4Rect.size.width, (int)a4Rect.size.height];

	prstr = [prstr stringByAppendingString: 
				@"%%EndComments\n\n%%BeginResource: procset GDraw_Procset 0 0\n"];
	
	fpath = [[[NSBundle mainBundle] bundlePath] stringByAppendingString:@"/Resources/GDProcSet"];
	prstr = [prstr stringByAppendingString: 
										[NSString stringWithContentsOfFile: fpath]];				
	
	prstr = [prstr stringByAppendingString: @"%%EndResource\n\n"];	
	
	if([fnames count]) {
		for(i = 0; i < [fnames count]; i++) {
			fname = [fnames objectAtIndex: i];
			prstr = [prstr stringByAppendingFormat: 
											@"%%%%BeginResource: font %@\n\n", fname]; 
			prstr = [prstr stringByAppendingString:  
							[[GDraw sharedgdraw] pfaDescriptionOfFont: fname]];
			prstr = [prstr stringByAppendingString: @"\n%%EndResource\n\n"];	
		}
	}

	prstr = [prstr stringByAppendingString: 
			@"\n\n%%BeginProlog\n%%EndProlog\n%%Page: 1 1\nGDraw_Procset begin\n\n"];	
	
	for(i = 0; i < [objects count]; i++) {
		obj = [objects objectAtIndex: i];
		str = [obj psDescription];
		if(str)
			prstr = [prstr stringByAppendingString: str];
	}
	prstr = [prstr stringByAppendingString: @"\nshowpage\n\n%%Trailer\n"];
	[prstr writeToFile: @"/tmp/gdraw.ps" atomically: NO];
}

- (NSArray *)usedFonts
{
	NSMutableArray *usedfonts;
	NSString *fname;
	id obj;
	int i, j;
	BOOL exist;
	
	usedfonts = [NSMutableArray arrayWithCapacity: 1];
	for(i = 0; i < [objects count]; i++) {
		exist = NO;
		obj = [objects objectAtIndex: i];
		if([obj isKindOfClass: [GDText class]]) {
			fname = [obj fontName];
			for(j = 0; j < [usedfonts count]; j++) {
				if([[usedfonts objectAtIndex: j] isEqualToString: fname]) {
					exist = YES;
					break;
				}
			}
			if(!exist)
				[usedfonts addObject: fname];
		}
	}
	
	return usedfonts;
}

- (BOOL)createObjectsFromDictionaryAtPath:(NSString *)aPath
{
	NSArray *keys;
	NSString *key;
	NSDictionary *dict, *objdict;
	BezierPathEditor *bzeditor;
	GDText *gdtext;
	int i;

	dict = [NSDictionary dictionaryWithContentsOfFile: aPath];
	if(!dict)
		return NO;
	
	keys = [dict allKeys];
	for(i = 0; i < [keys count]; i++) {
		key = [keys objectAtIndex: i];
		objdict = [dict objectForKey: key];
		if(!objdict)
			return NO;
		if([key rangeOfString: @"path"].length) {
			bzeditor = [[BezierPathEditor alloc] initFromData: objdict
														inView: self zoomFactor: zFactor];
			[objects addObject: bzeditor];
			[bzeditor release];
			edind = [objects count] -1;
		} else {
			gdtext = [[GDText alloc] initFromData: objdict
														inView: self zoomFactor: zFactor];
			[objects addObject: gdtext];
			[gdtext release];
		}
	}

	docName = [[NSString alloc] initWithString: aPath];
	[myWin setName: [docName lastPathComponent]];
	[myWin setSaved: YES];
	[[GDraw sharedgdraw] setWorkPath: [docName stringByDeletingLastPathComponent]];
	return YES;
}

- (NSString *)docName
{
	return docName;
}

- (void)addPathEditor
{
	BezierPathEditor *editor;
	
	editor = [[BezierPathEditor alloc] initInView: self zoomFactor: zFactor];
	[objects addObject: editor];
	[editor release];
	edind = [objects count] -1;
}

- (void)addTextAtPoint:(NSPoint)p
{
	GDText *gdtxt;
	int i;
	
	for(i = 0; i < [objects count]; i++)
		[[objects objectAtIndex: i] unselect];

	gdtxt = [[GDText alloc] initInView: self atPoint: p 
												zoomFactor: zFactor openEditor: YES];
	[objects addObject: gdtxt];
	[gdtxt select];
	[gdtxt release];
	[self setNeedsDisplay: YES];
	[myWin setSaved: NO];
}

- (NSArray *)duplicateObjects:(NSArray *)objs andMoveTo:(NSPoint)p
{
	id obj, duplObj;
	NSMutableArray *duplObjs;
	int i;

	duplObjs = [NSMutableArray arrayWithCapacity: 1];
	
	for(i = 0; i < [objs count]; i++) {
		obj = [objs objectAtIndex: i];
		if([obj isKindOfClass: [BezierPathEditor class]])
			duplObj = [(BezierPathEditor *)obj duplicate];
		else
			duplObj = [(GDText *)obj duplicate];
		[obj unselect];
		[duplObj selectAsGroup];
		[duplObj moveAddingCoordsOfPoint: p];	
		[objects addObject: duplObj];
		[duplObjs addObject: duplObj];
	}
	edind = [objects count] -1;
	[self setNeedsDisplay: YES];
	
	[doItAgain setArgument: &duplObjs atIndex: 2];
	[doItAgain setArgument: &p atIndex: 3];
	[doItAgain retainArguments];
	
	[myWin setSaved: NO];
	return duplObjs;
}

- (void)deleteSelectedObjects
{
	id obj;
	NSMutableArray *deleted;
	int i, count;

	deleted = [NSMutableArray arrayWithCapacity: 1];		

	count = [objects count];
	for(i = 0; i < count; i++) {
		obj = [objects objectAtIndex: i];
		if([obj isGroupSelected]) {
			[deleted addObject: obj];
			[objects removeObject: obj];
			count--;
			i--;
		}
	}
	if([deleted count]) {
		[delObjects addObject: deleted];
		[myWin setSaved: NO];
	}
	[self setNeedsDisplay: YES];	
	
	PREPAREUNDO(self, undoDeleteObjects);
}

- (void)undoDeleteObjects
{
	NSMutableArray *deleted;
	id obj;
	int i, index, count;
	
	for(i = 0; i < [objects count]; i++)
		[[objects objectAtIndex: i] unselect];

	if([delObjects count]) {
		index = [delObjects count] -1;
		deleted = [delObjects objectAtIndex: index];
		count = [deleted count];
		for(i = 0; i < count; i++) {
			obj = [deleted objectAtIndex: i];
			[objects addObject: obj];
			[deleted removeObject: obj];
			count--;
			i--;
		}
		[delObjects removeObjectAtIndex: index];
		[myWin setSaved: NO];
	}
	[self setNeedsDisplay: YES];
}

- (void)startDrawingAtPoint:(NSPoint)p
{
  	NSEvent *nextEvent;
	BezierPathEditor *editor;
	id obj;
	BOOL isneweditor = YES;
	int i;
	
	[myWin setSaved: NO];
	for(i = 0; i < [objects count]; i++) {
		obj = [objects objectAtIndex: i];
		if([obj isKindOfClass: [BezierPathEditor class]])
			if(![obj isdone])
				isneweditor = NO;
	}
	
	if(isneweditor) 
		for(i = 0; i < [objects count]; i++)
			[[objects objectAtIndex: i] unselect];

  	nextEvent = [[self window] nextEventMatchingMask:
    										NSLeftMouseUpMask | NSLeftMouseDraggedMask];
	[self verifyModifiersOfEvent: nextEvent];
	
  	if([nextEvent type] != NSLeftMouseDragged) {
		if(isneweditor) {
			[self addPathEditor];
			editor = [objects objectAtIndex: edind];
			[editor selectForEditing];
			[editor addControlAtPoint: p];
			[self setNeedsDisplay: YES];
			return;
		} else {
			editor = [objects objectAtIndex: edind];
			[editor selectForEditing];
			if(shiftclick)
				p = pointApplyingCostrainerToPoint(p, [[editor lastPoint] center]);
			[editor addLineToPoint: p];
			[self setNeedsDisplay: YES];
			return;
		}    	
  	} else {
		if(isneweditor) {
			[self addPathEditor];
			editor = [objects objectAtIndex: edind];
			[editor selectForEditing];
			[editor addControlAtPoint: p];
		} else {
			editor = [objects objectAtIndex: edind];
			[editor selectForEditing];
			if(shiftclick)
				p = pointApplyingCostrainerToPoint(p, [[editor lastPoint] center]);
			[editor addControlAtPoint: p];
		}
		[self setNeedsDisplay: YES];
		
  		do {
			p = [nextEvent locationInWindow];
  			p = [self convertPoint: p fromView: nil];
			if(shiftclick)
				p = pointApplyingCostrainerToPoint(p, [[editor lastPoint] center]);
	
			[editor addCurveWithBezierHandlePosition: p];

			[self setNeedsDisplay: YES];
		
    		nextEvent = [[self window] nextEventMatchingMask:
        									NSLeftMouseUpMask | NSLeftMouseDraggedMask];
			[self verifyModifiersOfEvent: nextEvent];								
		} while([nextEvent type] != NSLeftMouseUp);
		
		[editor confirmNewCurve];
		[self setNeedsDisplay: YES];
	}
}

- (void)selectObjectAtPoint:(NSPoint)p
{
	id obj;
	NSMutableArray *objs;
	int i;

	objs = [NSMutableArray arrayWithCapacity: 1];	

	for(i = 0; i < [objects count]; i++) {
		obj = [objects objectAtIndex: i];
		if([obj isKindOfClass: [BezierPathEditor class]]) {
			if([obj onPathBorder: p])
				[obj selectAsGroup];
			else 
				if(!shiftclick) 
					[obj unselect];
		} else {
			if([obj pointInBounds: p])
				[obj select];
			else 
				if(!shiftclick)
					[obj unselect];
		}
	}
	
	for(i = 0; i < [objects count]; i++) {
		obj = [objects objectAtIndex: i];
		if([obj isGroupSelected])
			[objs addObject: obj];
	}
	
	[self setNeedsDisplay: YES];
	
	if([objs count])	
		[self moveSelectedObjects: objs startingPoint: p];
}

- (void)editPathAtPoint:(NSPoint)p
{
	id obj;
	int i;

	for(i = 0; i < [objects count]; i++) {
		obj = [objects objectAtIndex: i];
		if([obj isKindOfClass: [BezierPathEditor class]]) {
			if([obj onPathBorder: p]) {
				[obj selectForEditing];
				[self setNeedsDisplay: YES];
				[self moveControlPointOfEditor: obj toPoint: p];
				return;
			} else {
				if([self moveBezierHandleOfEditor: obj toPoint: p]) 
					return;
				else 
					[obj unselect];
			}
		} else {
			[obj unselect];
		}
	}
	[self setNeedsDisplay: YES];
}

- (void)editTextAtPoint:(NSPoint)p
{
	id obj;
	int i;

	for(i = 0; i < [objects count]; i++)
		[[objects objectAtIndex: i] unselect];

	for(i = 0; i < [objects count]; i++) {
		obj = [objects objectAtIndex: i];
		if([obj isKindOfClass: [GDText class]]) {
			if([obj pointInBounds: p]) {
				[obj select];
				[self setNeedsDisplay: YES];
				[obj edit];
				[myWin setSaved: NO];
				return;
			}
		}
	}
	[self setNeedsDisplay: YES];
}

- (void)editSelectedText
{
	id obj;
	int i;

	for(i = 0; i < [objects count]; i++) {
		obj = [objects objectAtIndex: i];
		if([obj isKindOfClass: [GDText class]]) {
			if([obj isSelect]) {
				[obj edit];
				[myWin setSaved: NO];
			}
		}
	}
}

- (void)moveSelectedObjects:(NSArray *)objs startingPoint:(NSPoint)startp
{
	NSEvent *nextEvent;
	NSArray *moveobjs = nil;
	id obj;
	NSPoint p, op, diffp;
	BOOL dupl = NO;
	int i;
	
	nextEvent = [[self window] nextEventMatchingMask:
    										NSLeftMouseUpMask | NSLeftMouseDraggedMask];
  	if([nextEvent type] == NSLeftMouseDragged) {
		[myWin setSaved: NO];
		[self verifyModifiersOfEvent: nextEvent];
		op.x = startp.x;
		op.y = startp.y;
		
  		do {
			p = [nextEvent locationInWindow];
  			p = [self convertPoint: p fromView: nil];
			if(shiftclick)
				p = pointApplyingCostrainerToPoint(p, startp);
		
			if(altclick && !dupl) {
				moveobjs = [self duplicateObjects: objs andMoveTo: NSMakePoint(0, 0)];
				dupl = YES;
			} else if(!moveobjs) {
				moveobjs = [NSArray arrayWithArray: objs];
			}
			
			diffp.x = p.x - op.x;
			diffp.y = p.y - op.y;
			for(i = 0; i < [moveobjs count]; i++) {
				obj = [moveobjs objectAtIndex: i];
				[obj moveAddingCoordsOfPoint: diffp];
			}
			op.x = p.x;
			op.y = p.y;
			[self setNeedsDisplay: YES];
    		nextEvent = [[self window] nextEventMatchingMask:
        									NSLeftMouseUpMask | NSLeftMouseDraggedMask];
			[self verifyModifiersOfEvent: nextEvent];	
		} while([nextEvent type] != NSLeftMouseUp);

		if(dupl) {
			diffp.x = p.x - startp.x;
			diffp.y = p.y - startp.y;
			[self prepareDoItAgainWithSelector: @selector(duplicateObjects:andMoveTo:)
								owner: self target: self , &moveobjs, &diffp, nil];
		} else {
			diffp.x = startp.x - p.x;
			diffp.y = startp.y - p.y;
			PREPAREUNDO(self, undoMoveObjects: [moveobjs retain] moveBackTo: diffp);
		}
	}
}

- (void)undoMoveObjects:(NSArray *)objs moveBackTo:(NSPoint)p
{
	id obj;
	int i;

	for(i = 0; i < [objects count]; i++)
		[[objects objectAtIndex: i] unselect];

	for(i = 0; i < [objs count]; i++) {
		obj = [objs objectAtIndex: i];
		[obj selectAsGroup];
		[obj moveAddingCoordsOfPoint: p];
	}
	[self setNeedsDisplay: YES];
	[myWin setSaved: NO];
}

- (BOOL)moveControlPointOfEditor:(BezierPathEditor *)editor toPoint:(NSPoint)pos
{
	NSPoint p;
	
	p = [editor moveControlAtPoint: pos];
	if(p.x == pos.x && p.y == pos.y)
		return NO;
			
	PREPAREUNDO(editor, moveControlAtPoint: p toPoint: pos);

	[self setNeedsDisplay: YES];	
	[myWin setSaved: NO];		
	return YES;
}

- (BOOL)moveBezierHandleOfEditor:(BezierPathEditor *)editor toPoint:(NSPoint)pos
{
	NSPoint p;
	
	p = [editor moveBezierHandleAtPoint: pos];
	if(p.x == pos.x && p.y == pos.y)
		return NO;

	PREPAREUNDO(editor, moveBezierHandleAtPoint: p toPoint: pos);
			
	[self setNeedsDisplay: YES];
	[myWin setSaved: NO];			
	return YES;
}

- (void)subdividePathAtPoint:(NSPoint)p splitIt:(BOOL)split
{
	id obj;
	int i;

	for(i = 0; i < [objects count]; i++) {
		obj = [objects objectAtIndex: i];
		if([obj isKindOfClass: [BezierPathEditor class]]) {
			if([obj onPathBorder: p]) {
				[obj selectForEditing];
				[obj subdividePathAtPoint: p splitIt: split];
				[myWin setSaved: NO];
				break;
			}
		}
	}
}

- (void)editObjectsProperties
{
	NSWindow *epwin;
	GDPropsEditor *propsEditor;
	unsigned int style = NSTitledWindowMask;
	NSMutableDictionary *objProps = nil;
	NSDictionary *newProps;
	id obj;
	NSNumber *num;
	float *color, newcolor[4];
	int i, count, result;
	
	if(![objects count])
		return;
	
	for(i = 0; i < [objects count]; i++) {
		obj = [objects objectAtIndex: i];
		if([obj isSelect]) {
			objProps = [NSMutableDictionary dictionaryWithCapacity: 1];
			if([obj isKindOfClass: [BezierPathEditor class]]) {
				[objProps setObject: @"path" forKey: @"type"];
				
				num = [NSNumber numberWithFloat: [obj flatness]];
				[objProps setObject: num forKey: @"flatness"];
				num = [NSNumber numberWithInt: [obj lineJoin]];
				[objProps setObject: num forKey: @"linejoin"];
				num = [NSNumber numberWithInt: [obj lineCap]];
				[objProps setObject: num forKey: @"linecap"];
				num = [NSNumber numberWithFloat: [obj miterLimit]];
				[objProps setObject: num forKey: @"miterlimit"];
				num = [NSNumber numberWithFloat: [obj lineWidth]];
				[objProps setObject: num forKey: @"linewidth"];
			
				num = [NSNumber numberWithInt: [obj isStroked]];
				[objProps setObject: num forKey: @"stroked"];
				color = [obj strokeColor];
				num = [NSNumber numberWithFloat: color[0]];
				[objProps setObject: num forKey: @"strokecyan"];
				num = [NSNumber numberWithFloat: color[1]];
				[objProps setObject: num forKey: @"strokemagenta"];
				num = [NSNumber numberWithFloat: color[2]];
				[objProps setObject: num forKey: @"strokeyellow"];
				num = [NSNumber numberWithFloat: color[3]];
				[objProps setObject: num forKey: @"strokeblack"];
				num = [NSNumber numberWithFloat: [obj strokeAlpha]];
				[objProps setObject: num forKey: @"strokealpha"];
			
				num = [NSNumber numberWithInt: [obj isFilled]];
				[objProps setObject: num forKey: @"filled"];
				color = [obj fillColor];
				num = [NSNumber numberWithFloat: color[0]];
				[objProps setObject: num forKey: @"fillcyan"];
				num = [NSNumber numberWithFloat: color[1]];
				[objProps setObject: num forKey: @"fillmagenta"];
				num = [NSNumber numberWithFloat: color[2]];
				[objProps setObject: num forKey: @"fillyellow"];
				num = [NSNumber numberWithFloat: color[3]];
				[objProps setObject: num forKey: @"fillblack"];
				num = [NSNumber numberWithFloat: [obj fillAlpha]];
				[objProps setObject: num forKey: @"fillalpha"];
			} else {
				[objProps setObject: @"text" forKey: @"type"];
				
				num = [NSNumber numberWithInt: [obj isStroked]];
				[objProps setObject: num forKey: @"stroked"];
				color = [obj strokeColor];
				num = [NSNumber numberWithFloat: color[0]];
				[objProps setObject: num forKey: @"strokecyan"];
				num = [NSNumber numberWithFloat: color[1]];
				[objProps setObject: num forKey: @"strokemagenta"];
				num = [NSNumber numberWithFloat: color[2]];
				[objProps setObject: num forKey: @"strokeyellow"];
				num = [NSNumber numberWithFloat: color[3]];
				[objProps setObject: num forKey: @"strokeblack"];
				num = [NSNumber numberWithFloat: [obj strokeAlpha]];
				[objProps setObject: num forKey: @"strokealpha"];
			
				num = [NSNumber numberWithInt: [obj isFilled]];
				[objProps setObject: num forKey: @"filled"];
				color = [obj fillColor];
				num = [NSNumber numberWithFloat: color[0]];
				[objProps setObject: num forKey: @"fillcyan"];
				num = [NSNumber numberWithFloat: color[1]];
				[objProps setObject: num forKey: @"fillmagenta"];
				num = [NSNumber numberWithFloat: color[2]];
				[objProps setObject: num forKey: @"fillyellow"];
				num = [NSNumber numberWithFloat: color[3]];
				[objProps setObject: num forKey: @"fillblack"];
				num = [NSNumber numberWithFloat: [obj fillAlpha]];
				[objProps setObject: num forKey: @"fillalpha"];
			}
			break;
		}
	}
	
	if(!objProps)
		return;
		
	epwin = [[NSWindow alloc] initWithContentRect: NSMakeRect(0, 0, 500, 305)
						styleMask: style backing: NSBackingStoreRetained defer: NO];
	[epwin setTitle: @"Object Properties"];
	propsEditor = [[GDPropsEditor alloc] initWithFrame: NSMakeRect(0, 0, 500, 300)
											forDocView: self objectProperties: objProps];										
	[epwin setContentView: propsEditor];
	[epwin center];
	result = [propsEditor runModal];
	if(result == NSAlertDefaultReturn) {
		newProps = [propsEditor properties];
		count = 0;
		for(i = 0; i < [objects count]; i++) 
			if([[objects objectAtIndex: i] isGroupSelected])
				count++;
		if(count > 1) {
			result = NSRunAlertPanel(@"Alert", @"You are going to set the properties of many objects! Are you sure?", @"Ok", @"No", nil);
			if(result != NS_ALERTDEFAULT) {
				[epwin release];	
				return;
			}
		}
		
		for(i = 0; i < [objects count]; i++) {
			obj = [objects objectAtIndex: i];
			if([obj isGroupSelected]) {
				objProps = [NSMutableDictionary dictionaryWithCapacity: 1];
				if([obj isKindOfClass: [BezierPathEditor class]]) {		
					[obj setFlat: [[newProps objectForKey: @"flatness"] floatValue]];
					[obj setLineJoin: [[newProps objectForKey: @"linejoin"] intValue]];
					[obj setLineCap: [[newProps objectForKey: @"linecap"] intValue]];
					[obj setMiterLimit: [[newProps objectForKey: @"miterlimit"] floatValue]];
					[obj setLineWidth: [[newProps objectForKey: @"linewidth"] floatValue]];
				}
				[obj setStroked: (BOOL)[[newProps objectForKey: @"stroked"] intValue]];
				newcolor[0] = [[newProps objectForKey: @"strokecyan"] floatValue];
				newcolor[1] = [[newProps objectForKey: @"strokemagenta"] floatValue];
				newcolor[2] = [[newProps objectForKey: @"strokeyellow"] floatValue];
				newcolor[3] = [[newProps objectForKey: @"strokeblack"] floatValue];
				[obj setStrokeColor: newcolor];
				[obj setStrokeAlpha: [[newProps objectForKey: @"strokealpha"] floatValue]];
				[obj setFilled: (BOOL)[[newProps objectForKey: @"filled"] intValue]];
				newcolor[0] = [[newProps objectForKey: @"fillcyan"] floatValue];
				newcolor[1] = [[newProps objectForKey: @"fillmagenta"] floatValue];
				newcolor[2] = [[newProps objectForKey: @"fillyellow"] floatValue];
				newcolor[3] = [[newProps objectForKey: @"fillblack"] floatValue];
				[obj setFillColor: newcolor];
				[obj setFillAlpha: [[newProps objectForKey: @"fillalpha"] floatValue]];
			}
		}
		[myWin setSaved: NO];
	}
	
	[epwin release];
	[self setNeedsDisplay: YES];
}

- (void)moveSelectedObjectsToFront
{
	id obj = nil;
	int i;

	for(i = 0; i < [objects count]; i++) {
		if([[objects objectAtIndex: i] isGroupSelected]) {
			obj = [[objects objectAtIndex: i] retain];
			break;
		}
	}
	if(!obj)
		return;
		
	for(i = 0; i < [objects count]; i++) 
		if([objects objectAtIndex: i] != obj) 
			[[objects objectAtIndex: i] unselect];
		
	for(i = 0; i < [objects count]; i++) {
		if((obj == [objects objectAtIndex: i]) && (i + 1 < [objects count])) {
			[objects removeObjectAtIndex: i];
			[objects insertObject: obj atIndex: i + 1];
			[obj release];
			break;
		}
	}

	[myWin setSaved: NO];	
	[self setNeedsDisplay: YES];
}

- (void)moveSelectedObjectsToBack
{
	id obj = nil;
	int i;

	for(i = 0; i < [objects count]; i++) {
		if([[objects objectAtIndex: i] isGroupSelected]) {
			obj = [[objects objectAtIndex: i] retain];
			break;
		}
	}
	if(!obj)
		return;

	for(i = 0; i < [objects count]; i++) 
		if([objects objectAtIndex: i] != obj) 
			[[objects objectAtIndex: i] unselect];

	for(i = 0; i < [objects count]; i++) {
		if((obj == [objects objectAtIndex: i]) && ((i - 1) >= 0)) {
			[objects removeObjectAtIndex: i];
			[objects insertObject: obj atIndex: i - 1];
			[obj release];
			break;
		}
	}

	[myWin setSaved: NO];
	[self setNeedsDisplay: YES];
}

- (void)unselectOtherObjects:(id)anObject
{
	id obj;
	int i;

	if(shiftclick)
		return;
		
	for(i = 0; i < [objects count]; i++) {
		obj = [objects objectAtIndex: i];
		if(obj != anObject)
			[obj unselect];
	}
	
	[self setNeedsDisplay: YES];
}

- (void)zoomOnPoint:(NSPoint)p zoomOut:(BOOL)isout
{
	float orx, ory, szx, szy;
	NSRect vr;
	NSPoint pp;
	int i;

	if(isout) {
		zIndex--;
		if(zIndex < 0) {
			zIndex = 0;
			return;
		}
	} else {
		zIndex++;
		if(zIndex > 8) {
			zIndex = 8;
			return;
		}
	}

	zFactor = zFactors[zIndex];
		
	orx = a4Rect.origin.x * zFactor;
	ory = a4Rect.origin.y * zFactor;
	szx = a4Rect.size.width * zFactor;
	szy = a4Rect.size.height * zFactor;
	zmdRect = NSMakeRect(orx, ory, szx, szy);
	szx = pageRect.size.width * zFactor;
	szy = pageRect.size.height * zFactor;		

	pp.x = p.x * ([self frame].origin.x + pageRect.size.width) / [self frame].size.width;
	pp.y = p.y * ([self frame].origin.y + pageRect.size.height) / [self frame].size.height;
	vr = NSMakeRect(pp.x * zFactor - 200, pp.y * zFactor - 200, 400, 400);

	[self setFrame: NSMakeRect(0, 0, szx, szy)];
	[self scrollRectToVisible: vr];
	
	for(i = 0; i < [objects count]; i++) 
		[[objects objectAtIndex: i] setZoomFactor: zFactor];
	
	[[self window] display];
}

- (void)movePageFromHandPoint:(NSPoint)handpos
{
	NSEvent *nextEvent;
	NSPoint p, diffp;
	NSRect r;

	nextEvent = [[self window] nextEventMatchingMask:
    										NSLeftMouseUpMask | NSLeftMouseDraggedMask];
  	if([nextEvent type] == NSLeftMouseDragged) {
  		do {
			p = [nextEvent locationInWindow];
  			p = [self convertPoint: p fromView: nil];
			diffp.x = p.x - handpos.x;
			diffp.y = p.y - handpos.y;
			r = [self visibleRect];
			r = NSMakeRect(r.origin.x - diffp.x, r.origin.y - diffp.y, 
																r.size.width, r.size.height);
			[self scrollRectToVisible: r];
    		nextEvent = [[self window] nextEventMatchingMask:
        									NSLeftMouseUpMask | NSLeftMouseDraggedMask];
		} while([nextEvent type] != NSLeftMouseUp);
		[[self window] display];
	}
}

- (void)doCut
{
	[self doCopy];
	[self deleteSelectedObjects];
	[myWin setSaved: NO];
}

- (void)doCopy
{
  	NSMutableArray *types;
  	NSPasteboard *pboard;
	id obj;
	NSMutableArray *objsdesc;
	int i;

	objsdesc = [NSMutableArray arrayWithCapacity: 1];	
	for(i = 0; i < [objects count]; i++) {
		obj = [objects objectAtIndex: i];
		if([obj isGroupSelected])
			[objsdesc addObject: [obj objectDescription]];
	}
		
	if([objsdesc count]) {
		types = [NSMutableArray arrayWithObjects: @"GDObjectPboardType", nil];
		pboard = [NSPasteboard generalPasteboard];  
  		[pboard declareTypes: types owner: self];
		[pboard setString: [objsdesc description] forType: @"GDObjectPboardType"];
	}
}

- (void)doPaste
{
  	NSPasteboard *pboard;
	NSArray *types;
	NSArray *descriptions;
	NSDictionary *objdesc;
	id obj;
	NSString *str;
	int i;

	pboard = [NSPasteboard generalPasteboard];  
	types = [NSArray arrayWithObject: @"GDObjectPboardType"];
  	if([[pboard availableTypeFromArray: types] isEqualToString: @"GDObjectPboardType"]) {
		descriptions = (NSArray *)[[pboard stringForType: @"GDObjectPboardType"] propertyList];

		for(i = 0; i < [descriptions count]; i++) {
			objdesc = [descriptions objectAtIndex: i];
			str = [objdesc objectForKey: @"type"];
			if([str isEqualToString: @"path"]) 
				obj = [[BezierPathEditor alloc] initFromData: objdesc 
															inView: self zoomFactor: zFactor];
			else
				obj = [[GDText alloc] initFromData: objdesc 
															inView: self zoomFactor: zFactor];
			[objects addObject: obj];
			[obj selectAsGroup];	
			[obj release];	
		}
		[self setNeedsDisplay: YES];
		[myWin setSaved: NO];
	}
}

- (void)doUndo
{
	if([undoManager canUndo]) {
		[undoManager undoNestedGroup];
		[myWin setSaved: NO];
	}
}

- (void)doRedo
{
	if([undoManager canRedo]) {
		[undoManager redo];
		[myWin setSaved: NO];
	}
}

- (void)prepareDoItAgainWithSelector:(SEL)selector 
								owner:(id)owner target:(id)target , ...
{
	NSMethodSignature	*sign;
	va_list ap;
  	id arg;
	int i = 2;
	
	sign = [owner methodSignatureForSelector: selector];
	if(doItAgain)
		[doItAgain release];
	doItAgain = [[NSInvocation invocationWithMethodSignature: sign] retain];
	[doItAgain setTarget: target];
	[doItAgain setSelector: selector];
	
  	va_start(ap, target);
	while(1) {
		arg = va_arg(ap, id);
		if(!arg)
			break;
		[doItAgain setArgument: arg atIndex: i];
		i++;
	}
	va_end(ap);

	[doItAgain retainArguments];
}

- (void)verifyModifiersOfEvent:(NSEvent *)theEvent
{
	if([theEvent type] == NSLeftMouseDown 
								|| [theEvent type] == NSLeftMouseDragged) {
		if([theEvent modifierFlags] & NSShiftKeyMask)
			shiftclick = YES;
		else
			shiftclick = NO;
		if([theEvent modifierFlags] & NSCommandKeyMask)
			altclick = YES;
		else
			altclick = NO;
		if([theEvent modifierFlags] & NSControlKeyMask)
			ctrlclick = YES;
		else
			ctrlclick = NO;
	}
	
	if([theEvent type] == NSLeftMouseUp) {   
		if(!([theEvent modifierFlags] & NSShiftKeyMask))
			shiftclick = NO;
		if(!([theEvent modifierFlags] & NSCommandKeyMask))
			altclick = NO;
		if(!([theEvent modifierFlags] & NSControlKeyMask))
			ctrlclick = NO;			
	}
}

- (BOOL)shiftclick
{
	return shiftclick;
}

- (BOOL)altclick
{
	return altclick;
}

- (BOOL)ctrlclick
{
	return ctrlclick;
}

- (void)mouseDown:(NSEvent *)theEvent
{
	NSPoint p;
	int count = [theEvent clickCount];
	
	[self verifyModifiersOfEvent: theEvent];
	
  	p = [theEvent locationInWindow];
  	p = [self convertPoint: p fromView: nil];

	if(count == 1) {
 		switch([[GDraw sharedgdraw] currentToolType]) {
			case blackarrowtool:
				[self selectObjectAtPoint: p];
				break;
			case whitearrowtool:
				[self editPathAtPoint: p];
				break;
			case beziertool:
				[self startDrawingAtPoint: p];
				break;
			case texttool:
				[self addTextAtPoint: p];	
				break;
			case circletool:

				break;
			case rectangletool:

				break;
			case painttool:

				break;
			case penciltool:

				break;
			case rotatetool:

				break;
			case reducetool:

				break;
			case reflecttool:

				break;
			case scissorstool:
				if(altclick)
					[self subdividePathAtPoint: p splitIt: NO];
				else
					[self subdividePathAtPoint: p splitIt: YES];
				break;
			case handtool:
				[self movePageFromHandPoint: p];
				break;
			case magnifytool:
				if(altclick)
					[self zoomOnPoint: p zoomOut: YES];
				else 
					[self zoomOnPoint: p zoomOut: NO];
				break;
			default:
				break;
		}
	} else {
 		if([[GDraw sharedgdraw] currentToolType] == blackarrowtool)
			[self editTextAtPoint: p];
	}
}

- (void)mouseUp:(NSEvent *)theEvent
{
	[self verifyModifiersOfEvent: theEvent];
}

- (BOOL)performKeyEquivalent:(NSEvent *)theEvent
{
	NSString *commchar = [theEvent charactersIgnoringModifiers];

	if([commchar isEqualToString: @"t"]) {
		[self editSelectedText];
		return YES;
	}		
	if([commchar isEqualToString: @"z"]) {
		[self doUndo];
		return YES;
	}
	if([commchar isEqualToString: @"r"]) {
		[self doRedo];
		return YES;
	}
	if([commchar isEqualToString: @"d"]) {
		if(doItAgain) {
			[doItAgain invoke];
			[self setNeedsDisplay: YES];
			[myWin setSaved: NO];
		}	
		return YES;
	}
	
	return NO;
}

- (void)keyDown:(NSEvent *)theEvent
{
	unsigned short keyCode;
	NSRect vRect, hiddRect;
	NSPoint vPoint;
	float hiddRx, hiddRy, hiddRw, hiddRh;
	
	keyCode = [theEvent keyCode];

	if(keyCode == NSDeleteFunctionKey) {
		[self deleteSelectedObjects];
	} else if(keyCode == NSPageUpFunctionKey) {
		vRect = [self visibleRect];
		vPoint = vRect.origin;
		hiddRx = vPoint.x;
		hiddRy = vPoint.y + vRect.size.height;
		hiddRw = vRect.size.width;
		hiddRh = vRect.size.height;
		hiddRect = NSMakeRect(hiddRx, hiddRy, hiddRw, hiddRh);
		[self scrollRectToVisible: hiddRect];
	} else if(keyCode == NSPageDownFunctionKey) {
		vRect = [self visibleRect];
		vPoint = vRect.origin;
		hiddRx = vPoint.x;
		hiddRy = vPoint.y - vRect.size.height;
		hiddRw = vRect.size.width;
		hiddRh = vRect.size.height;
		hiddRect = NSMakeRect(hiddRx, hiddRy, hiddRw, hiddRh);
		[self scrollRectToVisible: hiddRect];
	}
}

- (BOOL)acceptsFirstResponder
{
	return YES;
}

- (BOOL)acceptsFirstMouse:(NSEvent *)theEvent
{
	return YES;
}

- (void)drawRect:(NSRect)rect
{
	int i;
	
	PSgsave();
	[[NSColor whiteColor] set];
	NSRectFill(rect);	

	[[NSColor lightGrayColor] set];
	PSmoveto(0, zmdRect.origin.y);
	PSlineto(rect.size.width, zmdRect.origin.y);
	PSmoveto(0, zmdRect.origin.y + zmdRect.size.height);
	PSlineto(rect.size.width, zmdRect.origin.y + zmdRect.size.height);
	PSmoveto(zmdRect.origin.x, 0);
	PSlineto(zmdRect.origin.x, rect.size.height);
	PSmoveto(zmdRect.origin.x + zmdRect.size.width, 0);
	PSlineto(zmdRect.origin.x + zmdRect.size.width, rect.size.height);
	PSstroke();
	PSgrestore();
	
	for(i = 0; i < [objects count]; i++) 
		[[objects objectAtIndex: i] Draw];
		
}

@end


