#include "docwin.h"
#include "gdraw.h"

@implementation DocWin

- (id)init
{
	NSScrollView *scrollView;
	NSRect rect;
	unsigned int style = NSTitledWindowMask | NSClosableWindowMask				
							| NSMiniaturizableWindowMask | NSResizableWindowMask;
	
	rect = NSMakeRect(0, 0, 720, [[NSScreen mainScreen] frame].size.height -60);
	self = [super initWithContentRect: rect styleMask: 
							style backing: NSBackingStoreRetained defer: NO];
	if(self) {
		name = [[NSString alloc] initWithString: @"Untitled"];
		[self setTitle: name];
		saved = YES;
		scrollView = [[[NSScrollView alloc] initWithFrame: rect] autorelease];
		[scrollView setHasHorizontalScroller: YES];
		[scrollView setHasVerticalScroller: YES]; 
		[scrollView setAutoresizingMask: NSViewHeightSizable | NSViewWidthSizable];

		docView = [[DocView alloc] initInWin: self];
		
		[scrollView setDocumentView: docView];	
		[[self contentView] addSubview: scrollView];
		
   	[[NSNotificationCenter defaultCenter] addObserver:self 
                					selector:@selector(windowExpose:) 
                						 name:@"WindowDidBecomeKey" 
                					  object:nil];
	}
	
	return self;
}

- (void)dealloc
{
	[docView release];
	[name release];
	[super dealloc];
}

- (DocView *)docView
{
	return docView;
}

- (NSString *)name
{
	return name;
}

- (void)windowExpose:(NSNotification *)aNotification
{
	id obj = [aNotification object];
	
	if(![obj isKindOfClass: [DocWin class]])
		return;
	
	if(self == (DocWin *)obj)
		[[GDraw sharedgdraw] setCurrentView: docView];
}

- (BOOL)windowShouldClose:(id)sender
{
	if([[GDraw sharedgdraw] removeWinFromWinList: self])
		return YES;
	return NO;
}

- (void)close
{
	if([[GDraw sharedgdraw] removeWinFromWinList: self])
		[super close];
}

- (void)setName:(NSString *)n
{
	if(![name isEqualToString: n]) {
		[name release];
		name = [[NSString alloc] initWithString: n];
	}
	if(saved)
		[self setTitle: name];
	else
		[self setTitle: [NSString stringWithFormat:@"%@ (modified)", name]];
}

- (void)setSaved:(BOOL)value
{
	if(value != saved) {
		saved = value;
		[self setName: name];
	}
}

- (BOOL)isSaved
{
	return saved;
}

@end
